# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable

import httpx

from ..types import (
    evaluation_get_run_params,
    evaluation_evaluate_params,
    evaluation_delete_run_params,
    evaluation_get_metric_params,
    evaluation_evaluate_run_params,
    evaluation_evaluate_dataset_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.evaluation_get_run_response import EvaluationGetRunResponse
from ..types.evaluation_evaluate_response import EvaluationEvaluateResponse
from ..types.evaluation_delete_run_response import EvaluationDeleteRunResponse
from ..types.evaluation_get_metric_response import EvaluationGetMetricResponse
from ..types.evaluation_evaluate_run_response import EvaluationEvaluateRunResponse
from ..types.evaluation_list_metrics_response import EvaluationListMetricsResponse
from ..types.evaluation_evaluate_dataset_response import EvaluationEvaluateDatasetResponse

__all__ = ["EvaluationResource", "AsyncEvaluationResource"]


class EvaluationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EvaluationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#accessing-raw-response-data-eg-headers
        """
        return EvaluationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#with_streaming_response
        """
        return EvaluationResourceWithStreamingResponse(self)

    def delete_run(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDeleteRunResponse:
        """
        Delete an evaluation run

        Args:
          id: The ID of the evaluation run to delete

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._delete(
            "/api/2/auto_eval/evaluation/delete_run",
            body=maybe_transform({"id": id}, evaluation_delete_run_params.EvaluationDeleteRunParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDeleteRunResponse,
        )

    def evaluate(
        self,
        *,
        ground_truth: List[str],
        input: List[str],
        metric: evaluation_evaluate_params.Metric,
        output: List[str],
        metadata: evaluation_evaluate_params.Metadata | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationEvaluateResponse:
        """Evaluate a metric on rows of data, returning scores for each row.

        Specify
        metric.id or metric.name to identify the metric.

        Args:
          metadata: Common metadata relevant to the application configuration from which all request
              inputs were derived. E.g. 'llm_model', 'chunk_size'

          project_id: The project where evaluation inference logs will be stored

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/evaluation/evaluate",
            body=maybe_transform(
                {
                    "ground_truth": ground_truth,
                    "input": input,
                    "metric": metric,
                    "output": output,
                    "metadata": metadata,
                    "project_id": project_id,
                },
                evaluation_evaluate_params.EvaluationEvaluateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationEvaluateResponse,
        )

    def evaluate_dataset(
        self,
        *,
        dataset_id: str,
        metrics: Iterable[evaluation_evaluate_dataset_params.Metric],
        experiment_id: str | NotGiven = NOT_GIVEN,
        metadata: evaluation_evaluate_dataset_params.Metadata | NotGiven = NOT_GIVEN,
        metric: evaluation_evaluate_dataset_params.Metric | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationEvaluateDatasetResponse:
        """Evaluate a metric on a dataset, returning scores for each example.

        Specify
        metric.id or metric.name to identify the metric. Persists results as an
        EvaluationRun for further capabilities.

        Args:
          dataset_id: The dataset to evaluate

          experiment_id: If specified, the evaluation run will be associated with this experiment

          metadata: Common metadata relevant to the application configuration from which all request
              inputs were derived. E.g. 'llm_model', 'chunk_size'

          metric: The metric to compute for the dataset. Use if only a single metric is required.
              For multiple metrics, use 'metrics'.

          project_id: The project where the evaluation run will be persisted

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/evaluation/evaluate_dataset",
            body=maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "metrics": metrics,
                    "experiment_id": experiment_id,
                    "metadata": metadata,
                    "metric": metric,
                    "project_id": project_id,
                },
                evaluation_evaluate_dataset_params.EvaluationEvaluateDatasetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationEvaluateDatasetResponse,
        )

    def evaluate_run(
        self,
        *,
        ground_truth: List[str],
        input: List[str],
        metrics: Iterable[evaluation_evaluate_run_params.Metric],
        output: List[str],
        experiment_id: str | NotGiven = NOT_GIVEN,
        metadata: evaluation_evaluate_run_params.Metadata | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationEvaluateRunResponse:
        """
        Similar to Evaluate, but persists results as an EvaluationRun for further
        capabilites.

        Args:
          experiment_id: If specified, the evaluation run will be associated with this experiment

          metadata: Common metadata relevant to the application configuration from which all request
              inputs were derived. E.g. 'llm_model', 'chunk_size' E.g. 'llm_model',
              'chunk_size'

          project_id: The project where the evaluation run will be persisted

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/evaluation/evaluate_run",
            body=maybe_transform(
                {
                    "ground_truth": ground_truth,
                    "input": input,
                    "metrics": metrics,
                    "output": output,
                    "experiment_id": experiment_id,
                    "metadata": metadata,
                    "project_id": project_id,
                },
                evaluation_evaluate_run_params.EvaluationEvaluateRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationEvaluateRunResponse,
        )

    def get_metric(
        self,
        *,
        metric: evaluation_get_metric_params.Metric,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationGetMetricResponse:
        """
        Get a specific evaluation metric by id or name.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/evaluation/get_metric",
            body=maybe_transform({"metric": metric}, evaluation_get_metric_params.EvaluationGetMetricParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationGetMetricResponse,
        )

    def get_run(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationGetRunResponse:
        """
        Get a specific evaluation run by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/evaluation/get_run",
            body=maybe_transform({"id": id}, evaluation_get_run_params.EvaluationGetRunParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationGetRunResponse,
        )

    def list_metrics(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationListMetricsResponse:
        """List all available evaluation metrics."""
        return self._post(
            "/api/2/auto_eval/evaluation/list_metrics",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationListMetricsResponse,
        )


class AsyncEvaluationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEvaluationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#with_streaming_response
        """
        return AsyncEvaluationResourceWithStreamingResponse(self)

    async def delete_run(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDeleteRunResponse:
        """
        Delete an evaluation run

        Args:
          id: The ID of the evaluation run to delete

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._delete(
            "/api/2/auto_eval/evaluation/delete_run",
            body=await async_maybe_transform({"id": id}, evaluation_delete_run_params.EvaluationDeleteRunParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDeleteRunResponse,
        )

    async def evaluate(
        self,
        *,
        ground_truth: List[str],
        input: List[str],
        metric: evaluation_evaluate_params.Metric,
        output: List[str],
        metadata: evaluation_evaluate_params.Metadata | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationEvaluateResponse:
        """Evaluate a metric on rows of data, returning scores for each row.

        Specify
        metric.id or metric.name to identify the metric.

        Args:
          metadata: Common metadata relevant to the application configuration from which all request
              inputs were derived. E.g. 'llm_model', 'chunk_size'

          project_id: The project where evaluation inference logs will be stored

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/evaluation/evaluate",
            body=await async_maybe_transform(
                {
                    "ground_truth": ground_truth,
                    "input": input,
                    "metric": metric,
                    "output": output,
                    "metadata": metadata,
                    "project_id": project_id,
                },
                evaluation_evaluate_params.EvaluationEvaluateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationEvaluateResponse,
        )

    async def evaluate_dataset(
        self,
        *,
        dataset_id: str,
        metrics: Iterable[evaluation_evaluate_dataset_params.Metric],
        experiment_id: str | NotGiven = NOT_GIVEN,
        metadata: evaluation_evaluate_dataset_params.Metadata | NotGiven = NOT_GIVEN,
        metric: evaluation_evaluate_dataset_params.Metric | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationEvaluateDatasetResponse:
        """Evaluate a metric on a dataset, returning scores for each example.

        Specify
        metric.id or metric.name to identify the metric. Persists results as an
        EvaluationRun for further capabilities.

        Args:
          dataset_id: The dataset to evaluate

          experiment_id: If specified, the evaluation run will be associated with this experiment

          metadata: Common metadata relevant to the application configuration from which all request
              inputs were derived. E.g. 'llm_model', 'chunk_size'

          metric: The metric to compute for the dataset. Use if only a single metric is required.
              For multiple metrics, use 'metrics'.

          project_id: The project where the evaluation run will be persisted

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/evaluation/evaluate_dataset",
            body=await async_maybe_transform(
                {
                    "dataset_id": dataset_id,
                    "metrics": metrics,
                    "experiment_id": experiment_id,
                    "metadata": metadata,
                    "metric": metric,
                    "project_id": project_id,
                },
                evaluation_evaluate_dataset_params.EvaluationEvaluateDatasetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationEvaluateDatasetResponse,
        )

    async def evaluate_run(
        self,
        *,
        ground_truth: List[str],
        input: List[str],
        metrics: Iterable[evaluation_evaluate_run_params.Metric],
        output: List[str],
        experiment_id: str | NotGiven = NOT_GIVEN,
        metadata: evaluation_evaluate_run_params.Metadata | NotGiven = NOT_GIVEN,
        project_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationEvaluateRunResponse:
        """
        Similar to Evaluate, but persists results as an EvaluationRun for further
        capabilites.

        Args:
          experiment_id: If specified, the evaluation run will be associated with this experiment

          metadata: Common metadata relevant to the application configuration from which all request
              inputs were derived. E.g. 'llm_model', 'chunk_size' E.g. 'llm_model',
              'chunk_size'

          project_id: The project where the evaluation run will be persisted

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/evaluation/evaluate_run",
            body=await async_maybe_transform(
                {
                    "ground_truth": ground_truth,
                    "input": input,
                    "metrics": metrics,
                    "output": output,
                    "experiment_id": experiment_id,
                    "metadata": metadata,
                    "project_id": project_id,
                },
                evaluation_evaluate_run_params.EvaluationEvaluateRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationEvaluateRunResponse,
        )

    async def get_metric(
        self,
        *,
        metric: evaluation_get_metric_params.Metric,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationGetMetricResponse:
        """
        Get a specific evaluation metric by id or name.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/evaluation/get_metric",
            body=await async_maybe_transform(
                {"metric": metric}, evaluation_get_metric_params.EvaluationGetMetricParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationGetMetricResponse,
        )

    async def get_run(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationGetRunResponse:
        """
        Get a specific evaluation run by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/evaluation/get_run",
            body=await async_maybe_transform({"id": id}, evaluation_get_run_params.EvaluationGetRunParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationGetRunResponse,
        )

    async def list_metrics(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationListMetricsResponse:
        """List all available evaluation metrics."""
        return await self._post(
            "/api/2/auto_eval/evaluation/list_metrics",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationListMetricsResponse,
        )


class EvaluationResourceWithRawResponse:
    def __init__(self, evaluation: EvaluationResource) -> None:
        self._evaluation = evaluation

        self.delete_run = to_raw_response_wrapper(
            evaluation.delete_run,
        )
        self.evaluate = to_raw_response_wrapper(
            evaluation.evaluate,
        )
        self.evaluate_dataset = to_raw_response_wrapper(
            evaluation.evaluate_dataset,
        )
        self.evaluate_run = to_raw_response_wrapper(
            evaluation.evaluate_run,
        )
        self.get_metric = to_raw_response_wrapper(
            evaluation.get_metric,
        )
        self.get_run = to_raw_response_wrapper(
            evaluation.get_run,
        )
        self.list_metrics = to_raw_response_wrapper(
            evaluation.list_metrics,
        )


class AsyncEvaluationResourceWithRawResponse:
    def __init__(self, evaluation: AsyncEvaluationResource) -> None:
        self._evaluation = evaluation

        self.delete_run = async_to_raw_response_wrapper(
            evaluation.delete_run,
        )
        self.evaluate = async_to_raw_response_wrapper(
            evaluation.evaluate,
        )
        self.evaluate_dataset = async_to_raw_response_wrapper(
            evaluation.evaluate_dataset,
        )
        self.evaluate_run = async_to_raw_response_wrapper(
            evaluation.evaluate_run,
        )
        self.get_metric = async_to_raw_response_wrapper(
            evaluation.get_metric,
        )
        self.get_run = async_to_raw_response_wrapper(
            evaluation.get_run,
        )
        self.list_metrics = async_to_raw_response_wrapper(
            evaluation.list_metrics,
        )


class EvaluationResourceWithStreamingResponse:
    def __init__(self, evaluation: EvaluationResource) -> None:
        self._evaluation = evaluation

        self.delete_run = to_streamed_response_wrapper(
            evaluation.delete_run,
        )
        self.evaluate = to_streamed_response_wrapper(
            evaluation.evaluate,
        )
        self.evaluate_dataset = to_streamed_response_wrapper(
            evaluation.evaluate_dataset,
        )
        self.evaluate_run = to_streamed_response_wrapper(
            evaluation.evaluate_run,
        )
        self.get_metric = to_streamed_response_wrapper(
            evaluation.get_metric,
        )
        self.get_run = to_streamed_response_wrapper(
            evaluation.get_run,
        )
        self.list_metrics = to_streamed_response_wrapper(
            evaluation.list_metrics,
        )


class AsyncEvaluationResourceWithStreamingResponse:
    def __init__(self, evaluation: AsyncEvaluationResource) -> None:
        self._evaluation = evaluation

        self.delete_run = async_to_streamed_response_wrapper(
            evaluation.delete_run,
        )
        self.evaluate = async_to_streamed_response_wrapper(
            evaluation.evaluate,
        )
        self.evaluate_dataset = async_to_streamed_response_wrapper(
            evaluation.evaluate_dataset,
        )
        self.evaluate_run = async_to_streamed_response_wrapper(
            evaluation.evaluate_run,
        )
        self.get_metric = async_to_streamed_response_wrapper(
            evaluation.get_metric,
        )
        self.get_run = async_to_streamed_response_wrapper(
            evaluation.get_run,
        )
        self.list_metrics = async_to_streamed_response_wrapper(
            evaluation.list_metrics,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["FineTuneJobSubmitParams", "FineTuneJobConfig"]


class FineTuneJobSubmitParams(TypedDict, total=False):
    fine_tune_job_config: Required[Annotated[FineTuneJobConfig, PropertyInfo(alias="fineTuneJobConfig")]]
    """The fine-tune job configuration."""

    job_id: Required[Annotated[str, PropertyInfo(alias="jobId")]]


class FineTuneJobConfig(TypedDict, total=False):
    baseline_model_id: Required[Annotated[str, PropertyInfo(alias="baselineModelId")]]
    """The ID for the model used as the starting point for training."""

    selected_columns: Required[Annotated[List[str], PropertyInfo(alias="selectedColumns")]]

    test_dataset_id: Required[Annotated[str, PropertyInfo(alias="testDatasetId")]]
    """The dataset to use for an unbiased evaluation of the model"""

    train_dataset_id: Required[Annotated[str, PropertyInfo(alias="trainDatasetId")]]
    """
    The dataset to use for training, with splits baked in or to be derived
    dynamically
    """

    description: str
    """Optional description for the job."""

    name: str
    """Optional name for the job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "DatasetGetViewParams",
    "Filter",
    "FilterAllOf",
    "FilterAllOfPredicate",
    "FilterAllOfPredicateNumericCriteria",
    "FilterAllOfPredicateStringCriteria",
    "FilterAllOfPredicateStringListCriteria",
    "FilterAnyOf",
    "FilterAnyOfPredicate",
    "FilterAnyOfPredicateNumericCriteria",
    "FilterAnyOfPredicateStringCriteria",
    "FilterAnyOfPredicateStringListCriteria",
]


class DatasetGetViewParams(TypedDict, total=False):
    dataset_file_id: Required[Annotated[str, PropertyInfo(alias="datasetFileId")]]
    """
    The ID of the (pinned) dataset file from which to retrieve content. Requests
    iterating over pages of results are recommended to use this pinned identifier
    after the first page in order to prevent any effects from a dataset changing
    between the queries.
    """

    dataset_id: Required[Annotated[str, PropertyInfo(alias="datasetId")]]
    """The ID of the dataset from which to retrieve content.

    When specified, gets data from the current file in the dataset.
    """

    filters: Required[Iterable[Filter]]

    after: int
    """Pagination: The index, by row-order, after which to query results."""

    get_last_page: Annotated[bool, PropertyInfo(alias="getLastPage")]

    limit: int
    """Pagination: The maximum number of results to return on this page."""

    next_page_cursor: Annotated[str, PropertyInfo(alias="nextPageCursor")]
    """A cursor for the next page in the pagination, if one exists."""

    order_by: Annotated[str, PropertyInfo(alias="orderBy")]
    """Column to order results by"""

    order_direction: Annotated[str, PropertyInfo(alias="orderDirection")]
    """Direction to order results ("asc" or "desc")"""

    previous_page_cursor: Annotated[str, PropertyInfo(alias="previousPageCursor")]
    """A cursor for the previous page in the pagination, if one exists."""

    use_datasets_service: Annotated[bool, PropertyInfo(alias="useDatasetsService")]


class FilterAllOfPredicateNumericCriteria(TypedDict, total=False):
    double_value: Required[Annotated[float, PropertyInfo(alias="doubleValue")]]

    int64_value: Required[Annotated[int, PropertyInfo(alias="int64Value")]]

    operator: Required[
        Literal[
            "OPERATOR_UNSPECIFIED",
            "OPERATOR_EQUALS",
            "OPERATOR_NOT_EQUALS",
            "OPERATOR_GREATER_THAN",
            "OPERATOR_GREATER_THAN_OR_EQUAL",
            "OPERATOR_LESS_THAN",
            "OPERATOR_LESS_THAN_OR_EQUAL",
        ]
    ]


class FilterAllOfPredicateStringCriteria(TypedDict, total=False):
    operator: Required[
        Literal[
            "OPERATOR_UNSPECIFIED",
            "OPERATOR_EQUALS",
            "OPERATOR_NOT_EQUALS",
            "OPERATOR_CONTAINS",
            "OPERATOR_STARTS_WITH",
            "OPERATOR_ENDS_WITH",
        ]
    ]

    value: Required[str]


class FilterAllOfPredicateStringListCriteria(TypedDict, total=False):
    operator: Required[Literal["OPERATOR_UNSPECIFIED", "OPERATOR_HAS_ANY", "OPERATOR_HAS_ALL", "OPERATOR_HAS_NONE"]]

    values: Required[List[str]]


class FilterAllOfPredicate(TypedDict, total=False):
    column_name: Required[Annotated[str, PropertyInfo(alias="columnName")]]

    numeric_criteria: Required[Annotated[FilterAllOfPredicateNumericCriteria, PropertyInfo(alias="numericCriteria")]]

    string_criteria: Required[Annotated[FilterAllOfPredicateStringCriteria, PropertyInfo(alias="stringCriteria")]]

    string_list_criteria: Required[
        Annotated[FilterAllOfPredicateStringListCriteria, PropertyInfo(alias="stringListCriteria")]
    ]


class FilterAllOf(TypedDict, total=False):
    predicates: Required[Iterable[FilterAllOfPredicate]]


class FilterAnyOfPredicateNumericCriteria(TypedDict, total=False):
    double_value: Required[Annotated[float, PropertyInfo(alias="doubleValue")]]

    int64_value: Required[Annotated[int, PropertyInfo(alias="int64Value")]]

    operator: Required[
        Literal[
            "OPERATOR_UNSPECIFIED",
            "OPERATOR_EQUALS",
            "OPERATOR_NOT_EQUALS",
            "OPERATOR_GREATER_THAN",
            "OPERATOR_GREATER_THAN_OR_EQUAL",
            "OPERATOR_LESS_THAN",
            "OPERATOR_LESS_THAN_OR_EQUAL",
        ]
    ]


class FilterAnyOfPredicateStringCriteria(TypedDict, total=False):
    operator: Required[
        Literal[
            "OPERATOR_UNSPECIFIED",
            "OPERATOR_EQUALS",
            "OPERATOR_NOT_EQUALS",
            "OPERATOR_CONTAINS",
            "OPERATOR_STARTS_WITH",
            "OPERATOR_ENDS_WITH",
        ]
    ]

    value: Required[str]


class FilterAnyOfPredicateStringListCriteria(TypedDict, total=False):
    operator: Required[Literal["OPERATOR_UNSPECIFIED", "OPERATOR_HAS_ANY", "OPERATOR_HAS_ALL", "OPERATOR_HAS_NONE"]]

    values: Required[List[str]]


class FilterAnyOfPredicate(TypedDict, total=False):
    column_name: Required[Annotated[str, PropertyInfo(alias="columnName")]]

    numeric_criteria: Required[Annotated[FilterAnyOfPredicateNumericCriteria, PropertyInfo(alias="numericCriteria")]]

    string_criteria: Required[Annotated[FilterAnyOfPredicateStringCriteria, PropertyInfo(alias="stringCriteria")]]

    string_list_criteria: Required[
        Annotated[FilterAnyOfPredicateStringListCriteria, PropertyInfo(alias="stringListCriteria")]
    ]


class FilterAnyOf(TypedDict, total=False):
    predicates: Required[Iterable[FilterAnyOfPredicate]]


class Filter(TypedDict, total=False):
    all_of: Required[Annotated[FilterAllOf, PropertyInfo(alias="allOf")]]

    any_of: Required[Annotated[FilterAnyOf, PropertyInfo(alias="anyOf")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["FineTuneJobListParams", "Filters"]


class FineTuneJobListParams(TypedDict, total=False):
    filters: Filters


class Filters(TypedDict, total=False):
    query: str
    """search query substring match for name and description"""

    status: Literal[
        "JOB_STATUS_UNSPECIFIED",
        "JOB_STATUS_QUEUED",
        "JOB_STATUS_RUNNING",
        "JOB_STATUS_COMPLETED",
        "JOB_STATUS_CANCELLED",
        "JOB_STATUS_FAILED",
    ]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import project_get_params, project_list_params, project_create_params, project_default_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.project_get_response import ProjectGetResponse
from ..types.project_list_response import ProjectListResponse
from ..types.project_create_response import ProjectCreateResponse
from ..types.project_default_response import ProjectDefaultResponse

__all__ = ["ProjectsResource", "AsyncProjectsResource"]


class ProjectsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#accessing-raw-response-data-eg-headers
        """
        return ProjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#with_streaming_response
        """
        return ProjectsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        description: str | NotGiven = NOT_GIVEN,
        organization_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProjectCreateResponse:
        """
        Create a new Project

        Args:
          name: Human-readable name for the project, if one exists.

          description: Human-readable description of the project, if one exists.

          organization_id: Organization to associate the project with, if applicable

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/project/create",
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "organization_id": organization_id,
                },
                project_create_params.ProjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectCreateResponse,
        )

    def list(
        self,
        *,
        filters: project_list_params.Filters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProjectListResponse:
        """
        List Projects.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/project/list",
            body=maybe_transform({"filters": filters}, project_list_params.ProjectListParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectListResponse,
        )

    def default(
        self,
        *,
        organization_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProjectDefaultResponse:
        """
        Get the default project for the user or organization

        Args:
          organization_id: If provided, will get the default project for the organization Otherwise, will
              retrieve the default project for the user or organization associated with the
              api key used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/project/get_default",
            body=maybe_transform({"organization_id": organization_id}, project_default_params.ProjectDefaultParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectDefaultResponse,
        )

    def get(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProjectGetResponse:
        """
        Get a Project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/2/auto_eval/project/get",
            body=maybe_transform({"id": id}, project_get_params.ProjectGetParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectGetResponse,
        )


class AsyncProjectsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProjectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProjectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProjectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lastmile-ai/lastmile-python#with_streaming_response
        """
        return AsyncProjectsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        description: str | NotGiven = NOT_GIVEN,
        organization_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProjectCreateResponse:
        """
        Create a new Project

        Args:
          name: Human-readable name for the project, if one exists.

          description: Human-readable description of the project, if one exists.

          organization_id: Organization to associate the project with, if applicable

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/project/create",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                    "organization_id": organization_id,
                },
                project_create_params.ProjectCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectCreateResponse,
        )

    async def list(
        self,
        *,
        filters: project_list_params.Filters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProjectListResponse:
        """
        List Projects.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/project/list",
            body=await async_maybe_transform({"filters": filters}, project_list_params.ProjectListParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectListResponse,
        )

    async def default(
        self,
        *,
        organization_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProjectDefaultResponse:
        """
        Get the default project for the user or organization

        Args:
          organization_id: If provided, will get the default project for the organization Otherwise, will
              retrieve the default project for the user or organization associated with the
              api key used

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/project/get_default",
            body=await async_maybe_transform(
                {"organization_id": organization_id}, project_default_params.ProjectDefaultParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectDefaultResponse,
        )

    async def get(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProjectGetResponse:
        """
        Get a Project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/2/auto_eval/project/get",
            body=await async_maybe_transform({"id": id}, project_get_params.ProjectGetParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProjectGetResponse,
        )


class ProjectsResourceWithRawResponse:
    def __init__(self, projects: ProjectsResource) -> None:
        self._projects = projects

        self.create = to_raw_response_wrapper(
            projects.create,
        )
        self.list = to_raw_response_wrapper(
            projects.list,
        )
        self.default = to_raw_response_wrapper(
            projects.default,
        )
        self.get = to_raw_response_wrapper(
            projects.get,
        )


class AsyncProjectsResourceWithRawResponse:
    def __init__(self, projects: AsyncProjectsResource) -> None:
        self._projects = projects

        self.create = async_to_raw_response_wrapper(
            projects.create,
        )
        self.list = async_to_raw_response_wrapper(
            projects.list,
        )
        self.default = async_to_raw_response_wrapper(
            projects.default,
        )
        self.get = async_to_raw_response_wrapper(
            projects.get,
        )


class ProjectsResourceWithStreamingResponse:
    def __init__(self, projects: ProjectsResource) -> None:
        self._projects = projects

        self.create = to_streamed_response_wrapper(
            projects.create,
        )
        self.list = to_streamed_response_wrapper(
            projects.list,
        )
        self.default = to_streamed_response_wrapper(
            projects.default,
        )
        self.get = to_streamed_response_wrapper(
            projects.get,
        )


class AsyncProjectsResourceWithStreamingResponse:
    def __init__(self, projects: AsyncProjectsResource) -> None:
        self._projects = projects

        self.create = async_to_streamed_response_wrapper(
            projects.create,
        )
        self.list = async_to_streamed_response_wrapper(
            projects.list,
        )
        self.default = async_to_streamed_response_wrapper(
            projects.default,
        )
        self.get = async_to_streamed_response_wrapper(
            projects.get,
        )
